unit OnChang;

interface

uses
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TBeforeChangeEvent =
     procedure(DeltaValue, CurrentValue, NewValue: integer;
               var Accept:boolean) of Object;
  TAfterChangeEvent =
     procedure(DeltaValue, PreviousValue, CurrentValue: integer) of Object;

  TIntOnChange = class(TComponent)
  private
    fBeforeChange: TBeforeChangeEvent;
    fAfterChange: TAfterChangeEvent;
    fPreviousValue_NO_ACCESS: integer;
    fCurrentValue_NO_ACCESS: integer;
    DeltaValue: integer;
  public
    MinValue,MaxValue,SumValue,CountValue: LongInt;
    StatsStartTime: TdateTime;
    procedure WriteValue(NewValue: integer);
    function ReadValue: integer;
    function ReadPreviousValue: integer;
    procedure ResetStats;
  published
    property BeforeChange: TBeforeChangeEvent read fBeforeChange write fBeforeChange;
    property AfterChange: TAfterChangeEvent read fAfterChange write fAfterChange;    
    constructor Create(aOwner: tComponent); override;
  end;

procedure Register;

implementation

constructor TIntOnChange.Create(aOwner: tComponent);
begin
  inherited Create(aOwner);
  fCurrentValue_NO_ACCESS:= 0;
  fPreviousValue_NO_ACCESS:= 0;
  DeltaValue:= 0;
  StatsStartTime:= Now;
  MinValue:= 0;
  MaxValue:= 0;
  SumValue:= 0;
  CountValue:= 0;
end;

procedure TIntOnChange.ResetStats;
begin
  MinValue:= 0;
  MaxValue:= 0;
  SumValue:= 0;
  CountValue:= 0;
  StatsStartTime:= Now;
end;

function TIntOnChange.ReadPreviousValue: integer;
begin
   Result:= fPreviousValue_NO_ACCESS;
end;

procedure TIntOnChange.WriteValue(NewValue: integer);
var AcceptOk: boolean;
begin
  AcceptOk:= True;
  if Assigned(fBeforeChange) then
      BeforeChange(NewValue-fCurrentValue_NO_ACCESS,
                   fCurrentValue_NO_ACCESS,
                   NewValue,AcceptOk);
  if not AcceptOk then
    Exit;
  fPreviousValue_NO_ACCESS:= fCurrentValue_NO_ACCESS;
  fCurrentValue_NO_ACCESS:= NewValue;
  DeltaValue:= fCurrentValue_NO_ACCESS-fPreviousValue_NO_ACCESS;
  if (MinValue = 0)  or (MinValue >  NewValue) then
    MinValue:= NewValue;
  if (MaxValue = 0) or (MaxValue <  NewValue)  then
    MaxValue:= NewValue;
  SumValue:= SumValue + NewValue;
  inc(CountValue);
  if Assigned(fAfterChange) then
      AfterChange(fCurrentValue_NO_ACCESS-fPreviousValue_NO_ACCESS,
                  fPreviousValue_NO_ACCESS,
                  fCurrentValue_NO_ACCESS);
end;

function TIntOnChange.ReadValue: integer;
begin
  Result:= fCurrentValue_NO_ACCESS;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TIntOnChange]);
end;

end.
 